from celery import shared_task


@shared_task
def update_endpoints_table(subject_identifiers: list[str] | None = None):
    from meta_analytics.dataframes import GlucoseEndpointsByDate

    if len(subject_identifiers) > 5:
        cls = GlucoseEndpointsByDate()
    else:
        cls = GlucoseEndpointsByDate(subject_identifiers=subject_identifiers)
    cls.run()
    return cls.to_model()



# Generated by Django 4.2.11 on 2024-07-27 00:12

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("sites", "0002_alter_domain_unique"),
        ("meta_reports", "0004_alter_patienthistorymissingbaselinecd4_table"),
    ]

    operations = [
        migrations.CreateModel(
            name="Endpoints",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField()),
                ("visit_code", models.IntegerField()),
                ("fasting", models.CharField(max_length=10)),
                ("fbg_datetime", models.DateTimeField()),
                ("fbg_value", models.FloatField()),
                ("ogtt_value", models.FloatField()),
                ("endpoint_label", models.CharField(max_length=25)),
                ("offstudy_datetime", models.DateTimeField(null=True)),
                ("offstudy_reason", models.CharField(max_length=250, null=True)),
                (
                    "site",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT, to="sites.site"
                    ),
                ),
            ],
            options={
                "verbose_name": "Endpoints (DM)",
                "verbose_name_plural": "Endpoints (DM)",
            },
        ),
    ]



from django.contrib import admin
from django.utils.translation import gettext_lazy as _
from django_audit_fields import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_consent_admin
from ..forms import SubjectConsentV1ExtForm
from ..models import SubjectConsentV1Ext
from .list_filters import AgreesListFilter


@admin.register(SubjectConsentV1Ext, site=meta_consent_admin)
class SubjectConsentV1ExtAdmin(
    ModelAdminSubjectDashboardMixin,
    SimpleHistoryAdmin,
):
    form = SubjectConsentV1ExtForm

    additional_instructions = (
        "This consent offers the participant the option to agree to "
        "extend clinic followup from the original 36 months to 48 months. "
    )

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_consent",
                    "report_datetime",
                    "agrees_to_extension",
                )
            },
        ),
        (
            "Review Questions",
            {
                "fields": (
                    "consent_reviewed",
                    "study_questions",
                    "assessment_score",
                    "consent_signature",
                    "consent_copy",
                ),
                "description": _("The following questions are directed to the interviewer."),
            },
        ),
        audit_fieldset_tuple,
    )

    list_display = (
        "subject_consent",
        "report_date",
        "agrees",
    )

    list_filter = (
        "report_datetime",
        AgreesListFilter,
    )

    radio_fields = {
        "agrees_to_extension": admin.VERTICAL,
        "consent_reviewed": admin.VERTICAL,
        "study_questions": admin.VERTICAL,
        "assessment_score": admin.VERTICAL,
        "consent_signature": admin.VERTICAL,
        "consent_copy": admin.VERTICAL,
    }

    def get_readonly_fields(self, request, obj=None) -> tuple[str, ...]:
        if obj:
            return ("subject_consent",)
        return ()

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "subject_consent":
            subject_identifier = request.GET.get("subject_identifier")
            kwargs["queryset"] = db_field.related_model.objects.filter(
                subject_identifier=subject_identifier
            )
        return super().formfield_for_foreignkey(db_field, request, **kwargs)

    @admin.display(description="Agrees")
    def agrees(self, obj):
        return obj.agrees_to_extension

    @admin.display(description="Report date")
    def report_date(self, obj):
        if obj:
            return obj.report_datetime.date()
        return None



# Generated by Django 5.0.1 on 2024-02-07 22:53

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0060_historicalicpreferral_locale_created_and_more"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalicpreferral",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectrefusal",
            name="site",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="icpreferral",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="subjectrefusal",
            name="site",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="+",
                to="sites.site",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="repeat_fasting_duration_minutes",
            field=models.IntegerField(
                blank=True, help_text="system calculated value", null=True
            ),
        ),
    ]



from django.contrib import admin
from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from edc_appointment.models import Appointment
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_visit_schedule.admin import ScheduleStatusListFilter
from edc_visit_schedule.constants import DAY1

from ...admin_site import meta_reports_admin
from ...models import PatientHistoryMissingBaselineCd4


@admin.register(PatientHistoryMissingBaselineCd4, site=meta_reports_admin)
class PatientHistoryMissingBaselineCd4Admin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    ordering = ["site", "subject_identifier"]
    list_display = [
        "dashboard",
        "subject_identifier",
        "cd4",
        "cd4_date",
        "site",
        "user_created",
        "user_modified",
        "modified",
    ]

    list_filter = [ScheduleStatusListFilter]

    search_fields = ["subject_identifier"]

    def dashboard(self, obj=None, label=None) -> str:
        kwargs = self.get_subject_dashboard_url_kwargs(obj)
        try:
            kwargs.update(
                appointment=str(
                    Appointment.objects.get(
                        subject_identifier=obj.subject_identifier,
                        visit_code=DAY1,
                        visit_code_sequence=0,
                    ).id
                )
            )
        except ObjectDoesNotExist:
            pass
        url = reverse(self.get_subject_dashboard_url_name(obj=obj), kwargs=kwargs)
        context = dict(title=_("Go to subject's dashboard@1000"), url=url, label=label)
        return render_to_string("edc_subject_dashboard/dashboard_button.html", context=context)



from copy import deepcopy
from random import choices

from dateutil.relativedelta import relativedelta
from django.test import TestCase
from edc_constants.constants import BLACK, FEMALE, MALE, NO, NOT_APPLICABLE, POS, YES
from edc_reportable import MILLIGRAMS_PER_DECILITER
from edc_utils.date import get_utcnow

from meta_screening.forms import (
    ScreeningPartOneForm,
    ScreeningPartThreeForm,
    ScreeningPartTwoForm,
)
from meta_screening.models.proxy_models import (
    ScreeningPartOne,
    ScreeningPartThree,
    ScreeningPartTwo,
)

from ..options import (
    get_part_one_eligible_options,
    get_part_three_eligible_options,
    get_part_two_eligible_options,
    now,
)


class ScreeningTestMixin:
    def complete_part_one(self, hospital_identifier=None, **options):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        hospital_identifier = hospital_identifier or "".join(
            [str(x) for x in choices(range(10), k=9)]  # nosec B311
        )
        data = {k: v for k, v in part_one_eligible_options.items()}
        data.update(hospital_identifier=hospital_identifier)
        data.update(**options or {})
        part_one_form = ScreeningPartOneForm(data=data)
        part_one_form.is_valid()
        self.assertEqual(part_one_form._errors, {})
        part_one_form.save()
        screening_identifier = part_one_form.instance.screening_identifier
        return ScreeningPartOne.objects.get(screening_identifier=screening_identifier)

    def complete_part_two(self, instance=None, **options):
        part_two_eligible_options = deepcopy(get_part_two_eligible_options())
        data = {k: v for k, v in part_two_eligible_options.items()}
        data.update(**options or {})
        part_two_form = ScreeningPartTwoForm(data=data, instance=instance)
        part_two_form.is_valid()
        self.assertEqual(part_two_form._errors, {})
        part_two_form.save()
        screening_identifier = part_two_form.instance.screening_identifier
        return part_two_form, ScreeningPartTwo.objects.get(
            screening_identifier=screening_identifier
        )

    def complete_part_three(self, instance=None, **options):
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(**options or {})
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertEqual(part_three_form._errors, {})
        part_three_form.save()
        screening_identifier = part_three_form.instance.screening_identifier
        return part_three_form, ScreeningPartThree.objects.get(
            screening_identifier=screening_identifier
        )


class TestForms(ScreeningTestMixin, TestCase):
    def setUp(self):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        part_two_eligible_options = deepcopy(get_part_two_eligible_options())
        self.screening_part_one = ScreeningPartOne(**part_one_eligible_options)
        self.screening_part_one.save()

        self.screening_identifier = self.screening_part_one.screening_identifier

        self.screening_part_two = ScreeningPartTwo.objects.get(
            screening_identifier=self.screening_identifier
        )
        for k, v in part_two_eligible_options.items():
            setattr(self.screening_part_two, k, v)
        self.screening_part_two.report_datetime = now
        self.screening_part_two.save()
        # part_three_eligible_options = deepcopy(
        #     get_part_three_eligible_options(report_datetime=obj.report_datetime)
        # )
        # self.data = part_three_eligible_options

    def test_screening_form_one_ok(self):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        data = {k: v for k, v in part_one_eligible_options.items()}
        data.update(hospital_identifier="12345678")
        form = ScreeningPartOneForm(data=data)
        form.is_valid()
        self.assertEqual(form._errors, {})

    def test_screening_form_two_ok(self):
        part_two_eligible_options = deepcopy(get_part_two_eligible_options())
        form = ScreeningPartTwoForm(
            data=part_two_eligible_options, instance=self.screening_part_two
        )
        form.is_valid()
        self.assertEqual(form._errors, {})

    def test_screening_form_three_ok(self):
        obj = ScreeningPartThree.objects.get(screening_identifier=self.screening_identifier)
        obj.part_two_report_datetime = now
        obj.save()
        part_three_eligible_options = deepcopy(
            get_part_three_eligible_options(report_datetime=obj.report_datetime)
        )
        form = ScreeningPartThreeForm(data=part_three_eligible_options, instance=obj)
        form.is_valid()
        self.assertEqual(form._errors, {})

    def test_screening_ok(self):
        part_one_eligible_options = deepcopy(get_part_one_eligible_options())
        part_two_eligible_options = deepcopy(get_part_two_eligible_options())
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_one_eligible_options.items()}
        data.update(hospital_identifier="12345111")
        form = ScreeningPartOneForm(data=data)
        form.is_valid()
        self.assertEqual(form._errors, {})
        form.save()

        screening_identifier = form.instance.screening_identifier

        instance = ScreeningPartTwo.objects.get(screening_identifier=screening_identifier)
        data = {k: v for k, v in part_two_eligible_options.items()}
        form = ScreeningPartTwoForm(data=part_two_eligible_options, instance=instance)
        form.is_valid()
        self.assertEqual(form._errors, {})
        form.save()

        instance = ScreeningPartThree.objects.get(
            screening_identifier=form.instance.screening_identifier
        )

        form = ScreeningPartThreeForm(data=part_three_eligible_options, instance=instance)
        form.is_valid()
        self.assertEqual(form._errors, {})
        form.save()

    def test_screening_two_urine_bhcg_pregnant_yes(self):
        instance = self.complete_part_one(gender=FEMALE, pregnant=YES)
        _, instance = self.complete_part_two(instance=instance)

        # if pregnant == YES, urine_bhcg_performed is applicable
        data = dict(
            part_three_report_datetime=instance.part_two_report_datetime
            + relativedelta(days=1),
            urine_bhcg_value=NOT_APPLICABLE,
            urine_bhcg_performed=NOT_APPLICABLE,
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_performed", part_three_form._errors)
        self.assertIn(
            "This field is applicable",
            part_three_form._errors.get("urine_bhcg_performed")[0],
        )

        # if pregnant == YES and urine_bhcg_performed=NO, urine_bhcg_value is NA
        data = dict(
            part_three_report_datetime=instance.part_two_report_datetime
            + relativedelta(days=1),
            urine_bhcg_value=POS,  # WRONG
            urine_bhcg_performed=NO,
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_value", part_three_form._errors)
        self.assertIn(
            "This field is not applicable",
            part_three_form._errors.get("urine_bhcg_value")[0],
        )

        # if pregnant == YES and urine_bhcg_performed=NO, urine_bhcg_value is NA OK
        data = dict(
            part_three_report_datetime=instance.part_two_report_datetime
            + relativedelta(days=1),
            urine_bhcg_performed=NO,
            urine_bhcg_value=NOT_APPLICABLE,
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertNotIn("urine_bhcg", part_three_form._errors)

        # if pregnant == YES and urine_bhcg_performed=YES,
        # urine_urine_bhcg_date is required
        data = dict(
            urine_bhcg_performed=YES,
            urine_bhcg_value=POS,
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn(
            "This field is required",
            part_three_form._errors.get("urine_bhcg_date")[0],
        )

        # if pregnant == YES and urine_bhcg_performed=YES and urine_bhcg is NEG
        # and urine_bhcg_date is now => OK
        data = dict(
            part_three_report_datetime=instance.part_two_report_datetime
            + relativedelta(days=1),
            urine_bhcg_value=POS,
            urine_bhcg_performed=YES,
            urine_bhcg_date=get_utcnow().date(),
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertNotIn("urine_bhcg", part_three_form._errors)

    def test_screening_two_urine_bhcg_pregnant_na(self):
        instance = self.complete_part_one(gender=FEMALE, pregnant=NOT_APPLICABLE)
        _, instance = self.complete_part_two(instance=instance)

        # if pregnant == NOT_APPLICABLE, urine_bhcg_performed is not applicable
        data = dict(
            urine_bhcg_performed=YES,
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_performed", part_three_form._errors)
        self.assertIn(
            "This field is not applicable",
            part_three_form._errors.get("urine_bhcg_performed")[0],
        )

        # if pregnant == NOT_APPLICABLE, urine_bhcg_performed is not applicable
        data = dict(
            urine_bhcg_performed=NOT_APPLICABLE,
            urine_bhcg_value=NOT_APPLICABLE,
            urine_bhcg_date=get_utcnow().date(),
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_date", part_three_form._errors)
        self.assertIn(
            "This field is not required",
            part_three_form._errors.get("urine_bhcg_date")[0],
        )

    def test_screening_two_urine_bhcg_pregnant_no(self):
        instance = self.complete_part_one(gender=FEMALE, pregnant=NO)
        _, instance = self.complete_part_two(instance=instance)
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        # gender F, pregnant=NO, urine_bhcg_performed should be yes/no
        data.update(
            dict(
                instance=instance,
                urine_bhcg_performed=NOT_APPLICABLE,
            )
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_performed", part_three_form._errors)
        self.assertIn(
            "This field is applicable",
            part_three_form._errors.get("urine_bhcg_performed")[0],
        )
        # if pregnant == NO, urine_bhcg_performed=NO, urine_bhcg is not
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(
            dict(
                instance=instance,
                urine_bhcg_value=POS,
                urine_bhcg_performed=NO,
            )
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_value", part_three_form._errors)
        self.assertIn(
            "This field is not applicable",
            part_three_form._errors.get("urine_bhcg_value")[0],
        )

        # if pregnant == NO, urine_bhcg_performed=YES, urine_bhcg_value is
        # not applicable
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(
            dict(
                instance=instance,
                urine_bhcg_value=NOT_APPLICABLE,
                urine_bhcg_performed=YES,
            )
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_value", part_three_form._errors)
        self.assertIn(
            "This field is applicable",
            part_three_form._errors.get("urine_bhcg_value")[0],
        )

        # if pregnant == NO, urine_bhcg_performed=NO, urine_bhcg_value is not
        # provided
        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(
            dict(
                instance=instance,
                pregnant=NO,
                urine_bhcg_value=None,
                urine_bhcg_performed=NO,
            )
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_value", part_three_form._errors)
        self.assertIn(
            "This field is required.",
            part_three_form._errors.get("urine_bhcg_value")[0],
        )

        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(
            dict(
                instance=instance,
                urine_bhcg_date=get_utcnow().date(),
                urine_bhcg_performed=NO,
            )
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_date", part_three_form._errors)
        self.assertIn(
            "This field is not required",
            part_three_form._errors.get("urine_bhcg_date")[0],
        )

        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(
            dict(
                instance=instance,
                urine_bhcg_value=POS,
                urine_bhcg_date=get_utcnow().date(),
                urine_bhcg_performed=YES,
            )
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()

    def test_screening_two_urine_bhcg_male_pregnant(self):
        instance = self.complete_part_one(gender=MALE, pregnant=NOT_APPLICABLE)
        _, instance = self.complete_part_two(instance=instance)

        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(dict(instance=instance, urine_bhcg_performed=YES))
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_performed", part_three_form._errors)
        self.assertIn(
            "This field is not applicable",
            part_three_form._errors.get("urine_bhcg_performed")[0],
        )

        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(dict(instance=instance, urine_bhcg_performed=NO))
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertIn("urine_bhcg_performed", part_three_form._errors)
        self.assertIn(
            "This field is not applicable",
            part_three_form._errors.get("urine_bhcg_performed")[0],
        )

        part_three_eligible_options = deepcopy(get_part_three_eligible_options())
        data = {k: v for k, v in part_three_eligible_options.items()}
        data.update(
            dict(
                instance=instance,
                urine_bhcg_performed=NOT_APPLICABLE,
                urine_bhcg_value=NOT_APPLICABLE,
            )
        )
        part_three_form = ScreeningPartThreeForm(data=data, instance=instance)
        part_three_form.is_valid()
        self.assertNotIn("urine_bhcg", part_three_form._errors)

    def test_screening_creatinine(self):
        instance = self.complete_part_one(hospital_identifier="9678281237")
        _, instance = self.complete_part_two(instance=instance)
        self.complete_part_three(
            instance=instance,
            verbose=True,
            ethnicity=BLACK,
            creatinine_value=1.5,
            creatinine_units=MILLIGRAMS_PER_DECILITER,
        )



from edc_reportable.data import africa, daids_july_2017

grading_data = {}
grading_data.update(**daids_july_2017.dummies)
grading_data.update(**daids_july_2017.chemistries)
grading_data.update(**daids_july_2017.hematology)

collection_name = "meta"
normal_data = africa.normal_data
reportable_grades = [3, 4]
reportable_grades_exceptions = {}

reference_range_options = dict(
    collection_name=collection_name,
    normal_data=africa.normal_data,
    grading_data=grading_data,
    reportable_grades=reportable_grades,
    reportable_grades_exceptions=reportable_grades_exceptions,
)



import pandas as pd
from django.contrib import admin, messages
from django.core.exceptions import FieldDoesNotExist
from django.db import models
from django.db.models import QuerySet
from django.template.loader import render_to_string
from django.utils.safestring import mark_safe
from edc_model_admin.dashboard import ModelAdminDashboardMixin
from edc_model_admin.list_filters import PastDateListFilter
from edc_model_admin.mixins import TemplatesModelAdminMixin
from edc_pdutils.actions import export_to_csv
from edc_qareports.modeladmin_mixins import QaReportModelAdminMixin
from edc_sites.admin import SiteModelAdminMixin
from edc_sites.admin.list_filters import SiteListFilter
from edc_utils import get_utcnow
from rangefilter.filters import DateRangeFilterBuilder

from meta_analytics.dataframes import get_last_imp_visits_df

from ..admin_site import meta_reports_admin
from ..models import LastImpRefill


class ImpVisitDateListFilter(PastDateListFilter):
    title = "IMP visit date"

    parameter_name = "imp_visit_date"
    field_name = "imp_visit_date"


class NextApptDateListFilter(PastDateListFilter):
    title = "Next appt date"

    parameter_name = "next_appt_date"
    field_name = "next_appt_date"


def update_report(modeladmin, request, queryset):
    now = get_utcnow()
    modeladmin.model.objects.all().delete()
    df = get_last_imp_visits_df()
    if not df.empty:
        data = [
            modeladmin.model(
                subject_identifier=row["subject_identifier"],
                site_id=row["site_id"],
                imp_visit_date=(
                    None if pd.isna(row["imp_visit_date"]) else row["imp_visit_date"]
                ),
                imp_visit_code=row["imp_visit_code"],
                next_appt_date=(
                    None if pd.isna(row["next_appt_datetime"]) else row["next_appt_datetime"]
                ),
                next_visit_code=(
                    None if pd.isna(row["next_visit_code"]) else row["next_visit_code"]
                ),
                days_since=row["days_since"].days,
                days_until=None if pd.isna(row["days_until"]) else row["days_until"].days,
                visit_code=(
                    None if pd.isna(row["imp_visit_code"]) else str(int(row["imp_visit_code"]))
                ),
                visit_code_sequence=(
                    None if pd.isna(row["imp_visit_code"]) else row["imp_visit_code"] % 1
                ),
                report_model=modeladmin.model._meta.label_lower,
                created=now,
            )
            for _, row in df.iterrows()
        ]
        created = len(modeladmin.model.objects.bulk_create(data))
        messages.success(request, "{} records were successfully created.".format(created))


@admin.register(LastImpRefill, site=meta_reports_admin)
class LastImpRefillAdmin(
    QaReportModelAdminMixin,
    SiteModelAdminMixin,
    ModelAdminDashboardMixin,
    TemplatesModelAdminMixin,
    admin.ModelAdmin,
):
    include_note_column = False

    change_list_title = "List of most recent IMP refills per subject"

    change_list_note = mark_safe(
        render_to_string("meta_reports/last_imp_refill/changelist_note.html")
    )  # nosec B308, B703

    actions = [update_report, export_to_csv]

    list_display = [
        "dashboard",
        "subject_identifier",
        "imp_visit_code",
        "imp_visit_date",
        "days_since",
        "next_visit_code",
        "next_appt_date",
        "days_until",
        "created",
    ]

    list_filter = [
        ("imp_visit_date", DateRangeFilterBuilder()),
        ("next_appt_date", DateRangeFilterBuilder()),
        "imp_visit_code",
        "next_visit_code",
        SiteListFilter,
    ]

    search_fields = ["subject_identifier"]

    def get_queryset(self, request) -> QuerySet:
        qs = super().get_queryset(request)
        if qs.count() == 0:
            update_report(self, request, None)
            qs = super().get_queryset(request)
        return qs

    def dataframe_to_model(self):
        now = get_utcnow()
        self.model.objects.all().delete()
        created = 0
        df = get_last_imp_visits_df()
        if not df.empty:
            data = [
                self.model(
                    subject_identifier=row["subject_identifier"],
                    site_id=row["site_id"],
                    imp_visit_date=row["imp_visit_date"],
                    imp_visit_code=row["imp_visit_code"],
                    next_appt_date=row["next_appt_datetime"],
                    next_visit_code=row["next_visit_code"],
                    days_since=row["days_since"],
                    days_until=row["days_until"],
                    visit_code=str(int(row["imp_visit_code"])),
                    visit_code_sequence=row["imp_visit_code"] % 1,
                    report_model=self.model._meta.label_lower,
                    created=now,
                )
                for _, row in df.iterrows()
            ]
            created = len(self.model.objects.bulk_create(data))
        return created

    def get_search_results(self, request, queryset, search_term):
        value = None
        fldname = None
        search_term = search_term.replace(" ", "")
        operators = {">=": "gte", "<=": "lte", "<": "lt", ">": "gt"}
        if op := [op for op in operators if op in search_term]:
            op = op[0]
            fldname, value = search_term.split(op)
            try:
                fldcls = self.model._meta.get_field(fldname)
            except FieldDoesNotExist:
                fldname = None
            else:
                if isinstance(fldcls, (models.IntegerField,)):
                    try:
                        value = int(value)
                    except ValueError:
                        value = None
                elif isinstance(fldcls, (models.FloatField,)):
                    try:
                        value = float(value)
                    except ValueError:
                        value = None
        if (
            value is not None
            and fldname
            and fldname in ["days_until", "days_since", "imp_visit_code", "next_visit_code"]
        ):
            queryset, use_distinct = super().get_search_results(request, queryset, None)
            queryset = queryset.filter(**{f"{fldname}__{operators[op]}": value})
        else:
            queryset, use_distinct = super().get_search_results(request, queryset, search_term)
        return queryset, use_distinct



# Generated by Django 5.1 on 2024-09-09 20:35
from pathlib import Path

import pandas as pd
from django.apps import apps as django_apps
from django.conf import settings
from django.db import migrations
from edc_utils import get_utcnow
from tqdm import tqdm


def populate_substitutions(apps, schema_editor):
    """import CSV file into pandas dataframe and use DF to
    populate the model.

    CSV file was sent to ERIK on 2024-09-09 in relation to
    a possible protocol violation where a subject was given the
    incorrect IMP. The CSV file lists `sid` where a bottle was not ready
    to dispense and a bottle of `dispensed_sid` was dispensed instead.
    """
    path = Path(settings.META_PHARMACY_RX_SUBSTITUTION_FILE).expanduser()
    df = pd.read_csv(path, header=1)
    df = df.rename(
        columns={"MISSING DRUG": "sid", "SUBSTITUTE": "dispensed_sid", "SITE": "site_id"}
    )
    df = df.rename(
        columns={"MISSING DRUG": "sid", "SUBSTITUTE": "dispensed_sid", "SITE": "site_id"}
    )
    df["sid"] = df["sid"].apply(pd.to_numeric, errors="coerce")
    df["dispensed_sid"] = df["dispensed_sid"].apply(pd.to_numeric, errors="coerce")
    df["site_id"] = df["site_id"].apply(pd.to_numeric, errors="coerce")
    now = get_utcnow()
    model_cls = django_apps.get_model("meta_pharmacy.substitutions")
    df_error = df[df["dispensed_sid"].isna()]
    df_error = df_error.reset_index()
    df_ok = df[df["dispensed_sid"].notna()]
    df_ok = df_ok.reset_index()
    data = [
        model_cls(
            row_index=row["index"],
            report_datetime=now,
            site_id=row["site_id"],
            sid=row["sid"],
            dispensed_sid=row["dispensed_sid"],
            user_created="erikvw",
            created=now,
            locale_created="en-gb",
        )
        for _, row in df_ok.iterrows()
    ]
    created = len(model_cls.objects.bulk_create(data))
    for obj in tqdm(model_cls.objects.all(), total=created):
        obj.save()
    if not df_error.empty:
        print(df_error[["index", "sid"]])


class Migration(migrations.Migration):

    dependencies = [
        ("meta_pharmacy", "0002_initial"),
    ]

    operations = [migrations.RunPython(populate_substitutions)]



# Generated by Django 5.0.8 on 2024-08-28 19:29

import django_db_views.migration_functions
import django_db_views.operations
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0040_auto_20240824_0412"),
    ]

    operations = []



# Generated by Django 5.2.3 on 2025-06-25 20:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0222_alter_historicalstudymedication_stock_codes_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="bloodresultsgludummy",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="bloodresultshba1c",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="bloodresultsins",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="bloodresultslft",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="bloodresultslipids",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="bloodresultsrft",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsgludummy",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalbloodresultshba1c",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsins",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslft",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalbloodresultslipids",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsrft",
            name="errors",
            field=models.TextField(blank=True, null=True),
        ),
    ]



from .eligibility import (
    MetaEligibility,
    SubjectScreeningEligibilityError,
    get_display_label,
)
from .eligibility_part_one import EligibilityPartOne
from .eligibility_part_three import EligibilityPartThreePhaseThree
from .eligibility_part_two import EligibilityPartTwo



from django.test import TestCase
from edc_reportable.models import GradingData, NormalData, ReferenceRangeCollection
from edc_reportable.utils import load_reference_ranges


class TestReportables(TestCase):
    def test_load_reference_ranges(self):

        from meta_labs.reportables import (
            collection_name,
            grading_data,
            normal_data,
            reportable_grades,
            reportable_grades_exceptions,
        )

        load_reference_ranges(
            collection_name=collection_name,
            grading_data=grading_data,
            normal_data=normal_data,
            reportable_grades=reportable_grades,
            reportable_grades_exceptions=reportable_grades_exceptions,
        )
        self.assertTrue(ReferenceRangeCollection.objects.filter(name=collection_name).exists())
        reference_range_collection = ReferenceRangeCollection.objects.get(name=collection_name)
        self.assertEqual(
            NormalData.objects.filter(
                reference_range_collection=reference_range_collection
            ).count(),
            82,
        )
        self.assertEqual(
            GradingData.objects.filter(
                reference_range_collection=reference_range_collection
            ).count(),
            174,
        )



# Generated by Django 5.1.2 on 2024-10-16 01:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0052_lastimpvisit"),
        ("sites", "0002_alter_domain_unique"),
    ]

    operations = [
        migrations.RenameModel(
            old_name="LastImpVisit",
            new_name="LastImpRefill",
        ),
        migrations.AlterModelOptions(
            name="lastimprefill",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "verbose_name": "Last IMP Refill",
                "verbose_name_plural": "Last IMP Refills",
            },
        ),
        migrations.RenameIndex(
            model_name="lastimprefill",
            new_name="meta_report_subject_bc8268_idx",
            old_name="meta_report_subject_4a9e15_idx",
        ),
    ]



from edc_listboard.views import SubjectListboardView as BaseSubjectListboardView

from meta_edc.meta_version import get_meta_version


class SubjectListboardView(BaseSubjectListboardView):
    listboard_model = "meta_consent.subjectconsentv1"
    navbar_selected_item = "consented_subject"

    def get_context_data(self, **kwargs) -> dict:
        kwargs.update(meta_version=get_meta_version())
        return super().get_context_data(**kwargs)

    def get_search_fields(self) -> list[str]:
        fields = super().get_search_fields()
        fields.append("identity__exact")
        return fields



# Generated by Django 4.2.11 on 2024-06-18 02:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="PatientHistoryMissingBaselineCd4",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("report_model", models.CharField(max_length=50)),
                ("subject_identifier", models.CharField(max_length=25)),
                ("created", models.DateTimeField()),
                ("visit_code", models.CharField(max_length=25)),
                ("visit_code_sequence", models.IntegerField()),
                ("cd4", models.IntegerField(verbose_name="Last CD4")),
                ("cd4_date", models.DateField(verbose_name="Date of last CD4")),
                ("user_created", models.CharField(max_length=25)),
                ("user_modified", models.CharField(max_length=25)),
                ("modified", models.DateTimeField()),
            ],
            options={
                "verbose_name": "Patient History: Missing Baseline Cd4",
                "verbose_name_plural": "Missing Baseline Cd4",
                "db_table": "patient_history_missing_baseline_cd4",
                "managed": False,
            },
        ),
        migrations.AlterModelOptions(
            name="unattendedthreeinrow",
            options={
                "managed": False,
                "verbose_name": "R100: Unattended appointments: Three in a row",
                "verbose_name_plural": "R100: Unattended appointments: Three in a row",
            },
        ),
        migrations.AlterModelOptions(
            name="unattendedthreeinrow2",
            options={
                "managed": False,
                "verbose_name": "R110: Unattended appointments: Three in a row (with missed)",
                "verbose_name_plural": "R110: Unattended appointments: Three in a row (with missed)",
            },
        ),
        migrations.AlterModelOptions(
            name="unattendedtwoinrow",
            options={
                "managed": False,
                "verbose_name": "R120: Unattended appointments: Two in a row",
                "verbose_name_plural": "R120: Unattended appointments: Two in a row",
            },
        ),
    ]



from edc_pharmacy.labels import (
    draw_bulk_stock_label_code128,
    draw_vertical_barcode_only_code128,
)
from edc_pylabels.site_label_configs import site_label_configs

from .labels import draw_label_for_subject_with_code128, draw_label_with_test_data

# create a label class to pass instead of a model

site_label_configs.register(
    "stock_vertical_barcode",
    draw_vertical_barcode_only_code128,
    "edc_pharmacy.stock",
    test_data_func=draw_label_with_test_data,
)

site_label_configs.register(
    "stock_bulk_barcode",
    draw_bulk_stock_label_code128,
    "edc_pharmacy.stock",
    test_data_func=draw_label_with_test_data,
)

site_label_configs.register(
    "patient_barcode",
    draw_label_for_subject_with_code128,
    "edc_pharmacy.stock",
    test_data_func=draw_label_with_test_data,
)



# Generated by Django 4.2.11 on 2024-05-17 14:31

from django.db import migrations, models
import edc_crf.model_mixins.crf_status_model_mixin


class Migration(migrations.Migration):

    dependencies = [
        ("meta_subject", "0205_historicalsubjectrequisition_crf_status_and_more"),
    ]

    operations = [
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="bloodresultsfbc",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="crf_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default=edc_crf.model_mixins.crf_status_model_mixin.get_crf_status_default,
                help_text="If some data is still pending, flag this CRF as incomplete",
                max_length=25,
                verbose_name="CRF status",
            ),
        ),
        migrations.AddField(
            model_name="historicalbloodresultsfbc",
            name="crf_status_comments",
            field=models.TextField(
                blank=True,
                help_text="for example, why some data is still pending",
                null=True,
                verbose_name="Any comments related to status of this CRF",
            ),
        ),
    ]



from django.db import models
from django.db.models import PROTECT, Index
from edc_consent.model_mixins import RequiresConsentFieldsModelMixin
from edc_crf.model_mixins import CrfStatusModelMixin
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_offstudy.model_mixins import OffstudyCrfModelMixin
from edc_sites.managers import CurrentSiteManager
from edc_sites.model_mixins import SiteModelMixin
from edc_utils import get_utcnow

from ..choices import FETAL_OUTCOMES
from .delivery import Delivery


class Manager(models.Manager):
    use_in_migrations = True

    def get_by_natural_key(
        self,
        birth_order,
        subject_identifier,
        visit_schedule_name,
        schedule_name,
        visit_code,
        visit_code_sequence,
    ):
        opts = dict(
            birth_order=birth_order,
            delivery__subject_visit__subject_identifier=subject_identifier,
            delivery__subject_visit__visit_schedule_name=visit_schedule_name,
            delivery__subject_visit__schedule_name=schedule_name,
            delivery__subject_visit__visit_code=visit_code,
            delivery__subject_visit__visit_code_sequence=visit_code_sequence,
        )
        return self.get(**opts)


class BirthOutcomes(
    CrfStatusModelMixin,
    SiteModelMixin,
    RequiresConsentFieldsModelMixin,
    OffstudyCrfModelMixin,
    BaseUuidModel,
):
    """A user model to capture birth outcomes.

    Related to the delivery model and presented as an inline
    in Admin.
    """

    delivery = models.ForeignKey(Delivery, on_delete=PROTECT)

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time",
        default=get_utcnow,
    )

    birth_order = models.IntegerField()

    birth_outcome = models.CharField(
        verbose_name="Outcome", max_length=25, choices=FETAL_OUTCOMES
    )

    birth_weight = models.IntegerField(
        verbose_name="Weight (gm)", help_text="gm", null=True, blank=True
    )

    objects = Manager()
    on_site = CurrentSiteManager()
    history = HistoricalRecords(inherit=True)

    def __str__(self):
        return f"{self.delivery} #{self.birth_order or '-'}"

    def natural_key(self) -> tuple:
        return (
            self.birth_order,
            self.delivery.subject_visit.subject_identifier,
            self.delivery.subject_visit.visit_code,
            self.delivery.subject_visit.visit_code_sequence,
            self.delivery.subject_visit.visit_schedule_name,
            self.delivery.subject_visit.visit_schedule,
        )

    def save(self, *args, **kwargs):
        self.report_datetime = self.delivery.report_datetime
        super().save(*args, **kwargs)

    @property
    def related_visit(self):
        return self.delivery.subject_visit

    @property
    def subject_identifier(self):
        return self.delivery.subject_identifier

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Birth Outcomes"
        verbose_name_plural = "Birth Outcomes"
        indexes = [Index(fields=["delivery", "birth_order"])]
        unique_together = ["delivery", "birth_order"]



from django.apps import apps as django_apps
from edc_action_item import Action, ActionWithNotification, site_action_items
from edc_action_item.site_action_items import AlreadyRegistered
from edc_adverse_event.constants import AE_INITIAL_ACTION
from edc_constants.constants import GRADE3, GRADE4, HIGH_PRIORITY, NEW, NONE, POS, YES
from edc_egfr.constants import EGFR_DROP_NOTIFICATION_ACTION
from edc_lab_results.action_items import BloodResultsFbcAction
from edc_lab_results.action_items import (
    BloodResultsGluAction as BaseBloodResultsGluAction,
)
from edc_lab_results.action_items import (
    BloodResultsHba1cAction,
    BloodResultsLftAction,
    BloodResultsLipidsAction,
)
from edc_lab_results.action_items import (
    BloodResultsRftAction as BaseBloodResultsRftAction,
)
from edc_ltfu.constants import LTFU_ACTION
from edc_visit_schedule.constants import OFFSCHEDULE_ACTION
from edc_visit_schedule.utils import is_baseline
from edc_visit_tracking.constants import MISSED_VISIT

from meta_prn.constants import (
    DM_REFFERAL_ACTION,
    OFFSCHEDULE_DM_REFERRAL_ACTION,
    OFFSCHEDULE_PREGNANCY_ACTION,
    PREGNANCY_NOTIFICATION_ACTION,
)

from .constants import (
    DELIVERY_ACTION,
    DM_FOLLOWUP_ACTION,
    FOLLOWUP_EXAMINATION_ACTION,
    MISSED_VISIT_ACTION,
    URINE_PREGNANCY_ACTION,
)


class MissedVisitAction(Action):
    name = MISSED_VISIT_ACTION
    priority = HIGH_PRIORITY
    display_name = "Missed Visits: LTFU"
    reference_model = "meta_subject.subjectvisitmissed"
    show_on_dashboard = True
    create_by_user = False

    def get_next_actions(self):
        """LTFU, 6 months off study medication using
        diff of report_datetime and refill_end_datetime.
        """
        next_actions = []
        subject_visit_model_cls = django_apps.get_model("meta_subject.subjectvisit")
        study_medication_model_cls = django_apps.get_model("meta_subject.studymedication")
        last_visit = (
            subject_visit_model_cls.objects.filter(subject_identifier=self.subject_identifier)
            .exclude(reason=MISSED_VISIT)
            .order_by("report_datetime")
            .last()
        )
        last_study_medication = (
            study_medication_model_cls.objects.filter(
                subject_visit__subject_identifier=self.subject_identifier
            )
            .order_by("refill_end_datetime")
            .last()
        )
        if (
            last_visit
            and last_study_medication
            and last_study_medication.refill_end_datetime
            and (last_study_medication.refill_end_datetime - last_visit.report_datetime).days
            >= 182
        ):
            next_actions = [LTFU_ACTION]
        return next_actions


class FollowupExaminationAction(Action):
    name = FOLLOWUP_EXAMINATION_ACTION
    priority = HIGH_PRIORITY
    display_name = "Followup Exam: AE"
    reference_model = "meta_subject.followupexamination"
    show_on_dashboard = True
    create_by_user = False

    def get_next_actions(self):
        next_actions = []
        if (
            self.reference_obj.symptoms_g3.exclude(name=NONE).count() > 0
            or self.reference_obj.symptoms_g4.exclude(name=NONE).count() > 0
            or self.reference_obj.any_other_problems_sae_grade in [GRADE3, GRADE4]
            or self.reference_obj.lactic_acidosis == YES
            or self.reference_obj.hepatomegaly == YES
        ) and not is_baseline(instance=self.reference_obj.subject_visit):
            next_actions.append(AE_INITIAL_ACTION)
        return next_actions


class UrinePregnancyAction(Action):
    name = URINE_PREGNANCY_ACTION
    priority = HIGH_PRIORITY
    display_name = "Urine Pregnancy Test"
    reference_model = "meta_subject.urinepregnancy"
    show_on_dashboard = True
    create_by_user = False

    def get_next_actions(self):
        next_actions = []
        if (self.reference_obj.bhcg_value == POS) and not is_baseline(
            instance=self.reference_obj.subject_visit
        ):
            next_actions.append(PREGNANCY_NOTIFICATION_ACTION)
        return next_actions


class BloodResultsRftAction(BaseBloodResultsRftAction):
    reference_model = "meta_subject.bloodresultsrft"

    def get_next_actions(self):
        next_actions = super().get_next_actions()
        if self.reference_obj.egfr_value is not None and self.reference_obj.egfr_value < 45.0:
            next_actions.append(OFFSCHEDULE_ACTION)
        return next_actions


class BloodResultsGluAction(BaseBloodResultsGluAction):
    reference_model = "meta_subject.bloodresultsgludummy"


class DeliveryAction(ActionWithNotification):
    name = DELIVERY_ACTION
    display_name = "Submit Delivery Form"
    notification_display_name = "Delivery Form"
    parent_action_names = [PREGNANCY_NOTIFICATION_ACTION]
    reference_model = "meta_subject.delivery"
    show_link_to_changelist = True
    show_link_to_add = False
    admin_site_name = "meta_subject_admin"
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        next_actions = [OFFSCHEDULE_PREGNANCY_ACTION]
        return next_actions


class EgfrDropNotificationAction(ActionWithNotification):
    name = EGFR_DROP_NOTIFICATION_ACTION
    display_name = "eGFR drop warning"
    notification_display_name = "eGFR drop warning"
    parent_action_names = []
    reference_model = "meta_subject.egfrdropnotification"
    show_link_to_changelist = True
    show_link_to_add = True
    admin_site_name = "meta_subject_admin"
    priority = HIGH_PRIORITY

    def close_action_item_on_save(self) -> bool:
        return self.reference_obj.report_status != NEW


class DmFollowAction(ActionWithNotification):
    name = DM_FOLLOWUP_ACTION
    display_name = "Diabetes Followup"
    notification_display_name = "Diabetes Followup"
    parent_action_names = [DM_REFFERAL_ACTION]
    reference_model = "meta_subject.dmfollowup"
    show_link_to_changelist = True
    show_link_to_add = False
    show_on_dashboard = False
    create_by_user = False
    create_by_action = True
    admin_site_name = "meta_subject_admin"
    priority = HIGH_PRIORITY

    def get_next_actions(self):
        next_actions = [OFFSCHEDULE_DM_REFERRAL_ACTION]
        return next_actions


def register_actions():
    for action_item_cls in [
        BloodResultsFbcAction,
        BloodResultsLipidsAction,
        BloodResultsLftAction,
        BloodResultsRftAction,
        BloodResultsHba1cAction,
        FollowupExaminationAction,
        MissedVisitAction,
        UrinePregnancyAction,
        DeliveryAction,
        EgfrDropNotificationAction,
        DmFollowAction,
    ]:
        try:
            site_action_items.register(action_item_cls)
        except AlreadyRegistered:
            del site_action_items.registry[action_item_cls.name]
            site_action_items.register(action_item_cls)


register_actions()



from django.db import models
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import COMPLETE, DONE, NOT_APPLICABLE, NOT_AVAILABLE, YES
from edc_glucose.model_mixins import (
    fasting_model_mixin_factory,
    ogtt_model_mixin_factory,
)
from edc_identifier.model_mixins import UniqueSubjectIdentifierFieldMixin
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_qareports.model_mixins import NoteModelMixin

NOTE_STATUSES = (
    (COMPLETE, "Complete"),
    (NOT_AVAILABLE, "Not available"),
)


class MissingOgttNote(
    fasting_model_mixin_factory(
        None,
        fasting=models.CharField(
            verbose_name="Did the participant fast?",
            max_length=15,
            choices=YES_NO_NA,
            default=NOT_APPLICABLE,
            blank=False,
        ),
    ),
    ogtt_model_mixin_factory("ogtt"),
    UniqueSubjectIdentifierFieldMixin,
    NoteModelMixin,
):
    """Model class to replace default `Note` model used with
    QA Report 'Screening: Missing OGTT'.
    """

    result_status = models.CharField(
        verbose_name="Is the OGTT result available", max_length=25, default=YES, choices=YES_NO
    )

    status = models.CharField(max_length=25, default=DONE, choices=NOTE_STATUSES)

    history = HistoricalRecords()

    def __str__(self) -> str:
        return f"{self._meta.verbose_name}: {self.subject_identifier}"

    def save(self, *args, **kwargs):
        if self.result_status == YES:
            self.status = COMPLETE
        else:
            self.status = NOT_AVAILABLE
        super().save(*args, **kwargs)

    class Meta(UniqueSubjectIdentifierFieldMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Screening: Missing OGTT Note"
        verbose_name_plural = "Screening: Missing OGTT Notes"



# Generated by Django 3.2.11 on 2022-01-28 14:19

import edc_model.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_prn", "0015_auto_20211104_1447"),
    ]

    operations = [
        migrations.AddField(
            model_name="endofstudy",
            name="clinical_withdrawal_reason",
            field=models.CharField(
                blank=True,
                choices=[
                    ("pregnancy", "Pregnancy"),
                    ("kidney_disease", "Development of chronic kidney disease"),
                    ("liver_disease", "Development of chronic liver disease"),
                    (
                        "intercurrent_illness",
                        "Intercurrent illness which prevents further treatment",
                    ),
                    ("investigator_decision", "Investigator decision"),
                    (
                        "OTHER",
                        "Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)",
                    ),
                ],
                max_length=25,
                null=True,
                verbose_name="If withdrawn for `clinical` reasons, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="clinical_withdrawal_reason_other",
            field=models.TextField(
                blank=True,
                max_length=500,
                null=True,
                verbose_name="If other `clinical` reason, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="last_seen_date",
            field=models.DateField(
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date patient was last seen",
            ),
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="toxicity_withdrawal_reason",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        "lactic_acidosis",
                        "Development of lactic acidosis or hyperlactatemia",
                    ),
                    ("hepatomegaly", "Development of hepatomegaly with steatosis"),
                    ("OTHER", "Other (specify below)"),
                ],
                max_length=25,
                null=True,
                verbose_name="If withdrawn for a `toxicity`, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="endofstudy",
            name="toxicity_withdrawal_reason_other",
            field=models.TextField(
                blank=True,
                max_length=500,
                null=True,
                verbose_name="If other `toxicity`, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="clinical_withdrawal_reason",
            field=models.CharField(
                blank=True,
                choices=[
                    ("pregnancy", "Pregnancy"),
                    ("kidney_disease", "Development of chronic kidney disease"),
                    ("liver_disease", "Development of chronic liver disease"),
                    (
                        "intercurrent_illness",
                        "Intercurrent illness which prevents further treatment",
                    ),
                    ("investigator_decision", "Investigator decision"),
                    (
                        "OTHER",
                        "Other condition that justifies the discontinuation of treatment in the clinician’s opinion (specify below)",
                    ),
                ],
                max_length=25,
                null=True,
                verbose_name="If withdrawn for `clinical` reasons, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="clinical_withdrawal_reason_other",
            field=models.TextField(
                blank=True,
                max_length=500,
                null=True,
                verbose_name="If other `clinical` reason, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="last_seen_date",
            field=models.DateField(
                null=True,
                validators=[edc_model.validators.date.date_not_future],
                verbose_name="Date patient was last seen",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="toxicity_withdrawal_reason",
            field=models.CharField(
                blank=True,
                choices=[
                    (
                        "lactic_acidosis",
                        "Development of lactic acidosis or hyperlactatemia",
                    ),
                    ("hepatomegaly", "Development of hepatomegaly with steatosis"),
                    ("OTHER", "Other (specify below)"),
                ],
                max_length=25,
                null=True,
                verbose_name="If withdrawn for a `toxicity`, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalendofstudy",
            name="toxicity_withdrawal_reason_other",
            field=models.TextField(
                blank=True,
                max_length=500,
                null=True,
                verbose_name="If other `toxicity`, please specify ...",
            ),
        ),
    ]



# Generated by Django 4.1.2 on 2022-11-30 19:48

from django.db import migrations, models
import django.db.models.deletion
import edc_model_fields.fields.other_charfield


class Migration(migrations.Migration):
    dependencies = [
        ("edc_adverse_event", "0010_auto_20220913_2139"),
        ("meta_ae", "0015_auto_20220907_0157"),
    ]

    operations = [
        migrations.RenameField(
            model_name="aetmg",
            old_name="narrative",
            new_name="investigator_narrative",
        ),
        migrations.RenameField(
            model_name="historicalaetmg",
            old_name="narrative",
            new_name="investigator_narrative",
        ),
        migrations.AddField(
            model_name="aetmg",
            name="investigator_ae_classification",
            field=models.ForeignKey(
                help_text="Only applicable if this investigator does not agree with the original AE report",
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                to="edc_adverse_event.aeclassification",
                verbose_name="Adverse Event (AE) Classification",
            ),
        ),
        migrations.AddField(
            model_name="aetmg",
            name="investigator_ae_classification_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If No, select a classification below",
                max_length=15,
                verbose_name="Does this investigator agree with the AE classification on the original AE report?",
            ),
        ),
        migrations.AddField(
            model_name="aetmg",
            name="investigator_ae_classification_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=250,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AddField(
            model_name="historicalaetmg",
            name="investigator_ae_classification",
            field=models.ForeignKey(
                blank=True,
                db_constraint=False,
                help_text="Only applicable if this investigator does not agree with the original AE report",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                related_name="+",
                to="edc_adverse_event.aeclassification",
                verbose_name="Adverse Event (AE) Classification",
            ),
        ),
        migrations.AddField(
            model_name="historicalaetmg",
            name="investigator_ae_classification_agreed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If No, select a classification below",
                max_length=15,
                verbose_name="Does this investigator agree with the AE classification on the original AE report?",
            ),
        ),
        migrations.AddField(
            model_name="historicalaetmg",
            name="investigator_ae_classification_other",
            field=edc_model_fields.fields.other_charfield.OtherCharField(
                blank=True,
                max_length=250,
                null=True,
                verbose_name="If other, please specify ...",
            ),
        ),
        migrations.AlterField(
            model_name="aefollowup",
            name="outcome",
            field=models.CharField(
                choices=[
                    ("continuing/update", "Continuing/Update"),
                    ("increase_from_g3", "Severity increased from Grade III"),
                    ("recovered", "Recovered/Resolved"),
                    ("recovering", "Recovering/Resolving at end of study"),
                    ("not_recovered", "Not Recovered/Resolved at end of study"),
                    ("LTFU", "Unknown/Lost to follow-up"),
                    ("recovered_with_sequelae", "Recovered with sequelae"),
                    ("dead", "Death"),
                    ("ae_withdrawn", "Adverse event report withdrawn after TMG review"),
                ],
                max_length=25,
            ),
        ),
        migrations.AlterField(
            model_name="historicalaefollowup",
            name="outcome",
            field=models.CharField(
                choices=[
                    ("continuing/update", "Continuing/Update"),
                    ("increase_from_g3", "Severity increased from Grade III"),
                    ("recovered", "Recovered/Resolved"),
                    ("recovering", "Recovering/Resolving at end of study"),
                    ("not_recovered", "Not Recovered/Resolved at end of study"),
                    ("LTFU", "Unknown/Lost to follow-up"),
                    ("recovered_with_sequelae", "Recovered with sequelae"),
                    ("dead", "Death"),
                    ("ae_withdrawn", "Adverse event report withdrawn after TMG review"),
                ],
                max_length=25,
            ),
        ),
    ]



from django.apps import apps as django_apps
from edc_utils import get_utcnow


def draw_label_with_test_data() -> dict:
    receive_model_cls = django_apps.get_model("edc_pharmacy.receive")
    receive_item_model_cls = django_apps.get_model("edc_pharmacy.receiveitem")
    location_model_cls = django_apps.get_model("edc_pharmacy.location")
    lot_model_cls = django_apps.get_model("edc_pharmacy.lot")
    container_type_model_cls = django_apps.get_model("edc_pharmacy.containertype")
    container_model_cls = django_apps.get_model("edc_pharmacy.container")
    container_type = container_type_model_cls(name="bottle")
    container = container_model_cls(
        name="bottle of 128", container_type=container_type, qty=128
    )
    return dict(
        subject_identifier="999-99999-9",
        code="A9B8C7",
        location=location_model_cls.objects.filter(site__isnull=False).first(),
        container=container,
        lot=lot_model_cls(lot_no="999999999", expiration_date=get_utcnow().date()),
        receive_item=receive_item_model_cls(
            receive=receive_model_cls(receive_identifier="99999999"),
            receive_item_identifier="99999999",
        ),
    )



from django.conf import settings
from edc_sites.single_site import SingleSite
from edc_sites.site import sites as site_sites

domain_suffix = settings.EDC_SITES_DOMAIN_SUFFIX
languages = ["sw", "en", "mas"]

all_sites = [
    SingleSite(
        10,
        "hindu_mandal",
        title="Hindu Mandal Hospital",
        country="tanzania",
        country_code="tz",
        domain=f"hindu-mandal.tz.{domain_suffix}",
        language_codes=languages,
    ),
    SingleSite(
        20,
        "amana",
        title="Amana Hospital",
        country="tanzania",
        country_code="tz",
        domain=f"amana.tz.{domain_suffix}",
        language_codes=languages,
    ),
    SingleSite(
        30,
        "temeke",
        title="Temeke Hospital",
        country="tanzania",
        country_code="tz",
        domain=f"temeke.tz.{domain_suffix}",
        language_codes=languages,
    ),
    SingleSite(
        40,
        "mwananyamala",
        title="Mwananyamala Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=languages,
        domain=f"mwananyamala.tz.{domain_suffix}",
    ),
    SingleSite(
        50,
        "mbagala",
        title="Mbagala Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=languages,
        domain=f"mbagala.tz.{domain_suffix}",
    ),
    SingleSite(
        60,
        "mnazi_moja",
        title="Mnazi Moja Hospital",
        country="tanzania",
        country_code="tz",
        language_codes=languages,
        domain=f"mnazi-moja.tz.{domain_suffix}",
    ),
]

site_sites.register(*all_sites)



from django.core.exceptions import ObjectDoesNotExist
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from edc_crf.crf_form_validator import CrfFormValidator
from edc_form_validators import INVALID_ERROR

from meta_reports.models import Endpoints
from meta_reports.tasks import update_endpoints_table


class DmEndpointFormValidator(CrfFormValidator):
    def clean(self):
        # recalc from endpoint report to confirm the endpoint has been reached
        update_endpoints_table(subject_identifiers=[self.subject_identifier])
        try:
            Endpoints.objects.get(subject_identifier=self.subject_identifier)
        except ObjectDoesNotExist:
            url = reverse("meta_reports_admin:meta_reports_glucosesummary_changelist")
            link = render_to_string(
                "meta_reports/columns/subject_identifier_column.html",
                {
                    "subject_identifier": self.subject_identifier,
                    "url": url,
                    "label": "Glucose Summary",
                },
            )
            self.raise_validation_error(
                {
                    "__all__": format_html(
                        "Subject has not reached the protocol endpoint. See {link}",
                        link=mark_safe(link),  # nosec B703, B308
                    )
                },
                INVALID_ERROR,
            )



from django.db import models
from edc_crf.model_mixins import CrfInlineModelMixin
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_model.validators import date_is_not_now, date_not_future
from edc_model_fields.fields import OtherCharField
from edc_sites.managers import CurrentSiteManager

from meta_lists.models import ArvRegimens

from .other_arv_regimens import OtherArvRegimens


class InlineModelManager(models.Manager):
    use_in_migrations = True

    def get_by_natural_key(
        self,
        other_arv_regimens,
        arv_regimen,
    ):
        return self.get(other_arv_regimens=other_arv_regimens, arv_regimen=arv_regimen)


class OtherArvRegimensDetail(CrfInlineModelMixin, BaseUuidModel):
    other_arv_regimens = models.ForeignKey(
        OtherArvRegimens,
        on_delete=models.PROTECT,
    )

    arv_regimen = models.ForeignKey(
        ArvRegimens,
        on_delete=models.PROTECT,
        related_name="other_arv_regimen",
        verbose_name="ARV Regimen",
        null=True,
        blank=False,
    )

    other_arv_regimen = OtherCharField(null=True, blank=True)

    arv_regimen_start_date = models.DateField(
        verbose_name="Start date",
        validators=[date_not_future, date_is_not_now],
        null=True,
        blank=True,
    )

    notes = models.CharField(verbose_name="Notes", max_length=100, null=True, blank=True)

    objects = InlineModelManager()
    on_site = CurrentSiteManager()
    history = HistoricalRecords(inherit=True)

    def natural_key(self) -> tuple:
        return self.other_arv_regimens, self.arv_regimen

    class Meta(CrfInlineModelMixin.Meta, BaseUuidModel.Meta):
        crf_inline_parent = "other_arv_regimens"
        verbose_name = "Other ARV Regimens Detail"
        verbose_name_plural = "Other ARV Regimens Detail"



# Generated by Django 3.2.11 on 2022-03-12 16:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_screening", "0038_auto_20220312_1929"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalicpreferral",
            name="fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartone",
            name="fbg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningpartthree",
            name="fbg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalscreeningparttwo",
            name="fbg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectscreening",
            name="fbg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
        migrations.AlterField(
            model_name="icpreferral",
            name="fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_fbg2_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="REPEAT FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="converted_fbg_value",
            field=models.DecimalField(
                decimal_places=4,
                help_text="mmol/L (system converted)",
                max_digits=8,
                null=True,
                verbose_name="FBG level",
            ),
        ),
        migrations.AlterField(
            model_name="subjectscreening",
            name="fbg2_units",
            field=models.CharField(
                blank=True,
                choices=[("mg/dL", "mg/dL"), ("mmol/L", "mmol/L (millimoles/L)")],
                max_length=15,
                null=True,
                verbose_name="FBG units",
            ),
        ),
    ]



# Generated by Django 3.2.11 on 2022-05-12 15:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0109_auto_20220415_1758"),
    ]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectvisit",
            name="document_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default="COMPLETE",
                help_text="If some data is still pending, flag as incomplete",
                max_length=25,
                verbose_name="Document status",
            ),
        ),
        migrations.AddField(
            model_name="subjectvisit",
            name="document_status",
            field=models.CharField(
                choices=[
                    ("INCOMPLETE", "Incomplete (some data pending)"),
                    ("COMPLETE", "Complete"),
                ],
                default="COMPLETE",
                help_text="If some data is still pending, flag as incomplete",
                max_length=25,
                verbose_name="Document status",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectvisit",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                    ("N/A", "Not applicable (if missed)"),
                ],
                default="alive",
                help_text="If YES, submit Death report",
                max_length=10,
                null=True,
                verbose_name="Participant's survival status",
            ),
        ),
        migrations.AlterField(
            model_name="subjectvisit",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                    ("N/A", "Not applicable (if missed)"),
                ],
                default="alive",
                help_text="If YES, submit Death report",
                max_length=10,
                null=True,
                verbose_name="Participant's survival status",
            ),
        ),
    ]



# TODO: urine_bhcg form (probably not necessary)
# TODO: if pos, take of study drug and estimate
#  delivery date for the pregnancy outcomes form. See Form 25/26
from django.apps import apps as django_apps
from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from django_crypto_fields.fields import EncryptedTextField
from edc_constants.choices import YES_NO, YES_NO_NA
from edc_constants.constants import NOT_APPLICABLE, PATIENT, YES
from edc_model.models import BaseUuidModel
from edc_model.validators import datetime_not_future
from edc_model_fields.fields import OtherCharField
from edc_protocol.validators import datetime_not_before_study_start
from edc_utils import get_utcnow

from ..choices import (
    DELIVERY_INFO_SOURCE,
    DELIVERY_INFORMANT_RELATIONSHIP,
    DELIVERY_LOCATIONS,
    MATERNAL_OUTCOMES,
)
from ..constants import DELIVERY_ACTION
from ..model_mixins import CrfWithActionModelMixin


class DeliveryError(Exception):
    pass


class Delivery(
    CrfWithActionModelMixin,
    BaseUuidModel,
):
    """A user model to capture delivery and birth outcomes.

    See also in signals `update_pregnancy_notification_on_delivery_post_save`
    """

    action_name = DELIVERY_ACTION

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time", default=get_utcnow
    )

    info_available = models.CharField(
        verbose_name="Were you able to obtain a report on the delivery?",
        max_length=5,
        choices=YES_NO,
        default=YES,
        help_text="If NO, please explain below",
    )

    info_not_available_reason = models.TextField(
        verbose_name="If the report was not available, please explain?",
        null=True,
        blank=True,
    )

    info_source = models.CharField(
        verbose_name="Who / what is the MAIN source of this information?",
        max_length=25,
        choices=DELIVERY_INFO_SOURCE,
        default=PATIENT,
    )

    info_source_other = EncryptedTextField(
        verbose_name=(
            "If not reported from study participant, please give "
            "the name and contact details of the informant."
        ),
        null=True,
        blank=True,
    )

    informant_relation = models.CharField(
        verbose_name="Informants relationship to the participant?",
        max_length=25,
        choices=DELIVERY_INFORMANT_RELATIONSHIP,
        default=NOT_APPLICABLE,
    )

    informant_relation_other = OtherCharField()

    delivery_datetime = models.DateTimeField(
        verbose_name="Date and time of delivery :",
        help_text="If TIME unknown, estimate",
        validators=[
            datetime_not_future,
            datetime_not_before_study_start,
        ],
        null=True,
    )

    delivery_time_estimated = models.CharField(
        verbose_name="Is the delivery TIME estimated?", max_length=3, choices=YES_NO_NA
    )

    delivery_location = models.CharField(
        verbose_name="Where did the delivery occur?",
        max_length=25,
        choices=DELIVERY_LOCATIONS,
        default=NOT_APPLICABLE,
    )

    delivery_location_other = OtherCharField()

    delivery_location_name = models.CharField(
        verbose_name=(
            "If delivery occurred at a `hospital` or `clinic`, "
            "please give name of the facility"
        ),
        max_length=150,
        null=True,
        blank=True,
    )

    delivery_ga = models.IntegerField(
        verbose_name="Gestational age at delivery",
        validators=[MinValueValidator(1), MaxValueValidator(45)],
        null=True,
    )

    maternal_outcome = models.CharField(
        verbose_name="What was the maternal outcome of the pregnancy?",
        max_length=50,
        choices=MATERNAL_OUTCOMES,
        default=NOT_APPLICABLE,
    )

    gm_treated = models.CharField(
        verbose_name="Was the participant treated for gestational diabetes?",
        max_length=5,
        choices=YES_NO_NA,
        default=NOT_APPLICABLE,
    )

    fetal_outcome_count = models.IntegerField(
        verbose_name="Number of births / fetal / neonatal outcomes",
        help_text="Each to be reported individually below",
        null=True,
    )

    def save(self, *args, **kwargs):
        pregnancy_notification_model_cls = django_apps.get_model(
            "meta_prn.pregnancynotification"
        )
        if (
            not self.id
            and not pregnancy_notification_model_cls.objects.filter(
                subject_identifier=self.subject_identifier,
                delivered=False,
            ).exists()
        ):
            raise DeliveryError(
                f"Invalid. A {pregnancy_notification_model_cls._meta.verbose_name} "
                "cannot be found. Perhaps catch this in the form."
            )
        super().save(*args, **kwargs)

    class Meta(CrfWithActionModelMixin.Meta, BaseUuidModel.Meta):
        verbose_name = "Delivery"
        verbose_name_plural = "Delivery"
        unique_together = ["subject_visit", "delivery_datetime"]



from django.db import models
from edc_constants.choices import POS_NEG_NA, YES_NO
from edc_model import models as edc_models

from ..constants import URINE_PREGNANCY_ACTION
from ..model_mixins import CrfWithActionModelMixin


class UrinePregnancy(CrfWithActionModelMixin, edc_models.BaseUuidModel):
    action_name = URINE_PREGNANCY_ACTION
    tracking_identifier_prefix = "UP"

    performed = models.CharField(
        verbose_name="Was the urine pregnancy test performed?",
        max_length=15,
        choices=YES_NO,
    )

    not_performed_reason = models.CharField(
        verbose_name="If NO, provide reason", max_length=150, null=True, blank=True
    )

    assay_date = models.DateField(verbose_name="Urine βhCG date", blank=True, null=True)

    bhcg_value = models.CharField(
        verbose_name="Urine βhCG result",
        max_length=25,
        choices=POS_NEG_NA,
    )

    notified = models.BooleanField(
        default=False,
        editable=False,
        help_text="Auto-updated by Pregnancy Notification PRN form",
    )

    notified_datetime = models.DateTimeField(
        null=True,
        editable=False,
        help_text="Auto-updated by Pregnancy Notification PRN form",
    )

    class Meta(CrfWithActionModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Urine Pregnancy"



CLINICAL_WITHDRAWAL = "clinical_withdrawal"
CLINICIAN = "clinician"
COMPLETED_FOLLOWUP_48 = "completed_followup_48"
DEATH_REPORT_ACTION = "submit_death_report"
DEVIATION = "protocol_deviation"
DM_REFFERAL_ACTION = "dm_refferal_action"
INVESTIGATOR = "investigator"
INVESTIGATOR_DECISION = "investigator_decision"
LATE_EXCLUSION = "late_exclusion"
LOSS_TO_FOLLOWUP_ACTION = "submit_loss_to_followup"
OFFSCHEDULE_DM_REFERRAL_ACTION = "offschedule_dm_referral_action"
OFFSCHEDULE_POSTNATAL_ACTION = "offschedule_postnatal_action"
OFFSCHEDULE_PREGNANCY_ACTION = "offschedule_pregnancy_action"
OFFSTUDY_MEDICATION_ACTION = "withdrawal_study_medication_action"
OTHER_RX_DISCONTINUATION = "other_treatment_discontinuation"
PREGNANCY_NOTIFICATION_ACTION = "pregancy_notification_action"
PROTOCOL_DEVIATION_VIOLATION_ACTION = "submit-protocol-deviation-violation"
REFERRAL = "referral"
SAE = "sae"
UNBLINDING_REQUEST_ACTION = "unblinding-request"
UNBLINDING_REVIEW_ACTION = "unblinding-review"
VIOLATION = "protocol_violation"



from dateutil.relativedelta import relativedelta
from django.core.exceptions import ObjectDoesNotExist
from django.test import TestCase
from edc_constants.constants import YES
from edc_pharmacy.exceptions import PrescriptionAlreadyExists, StudyMedicationError
from edc_pharmacy.models import DosageGuideline, Formulation, Medication, RxRefill
from edc_pharmacy.prescribe import create_prescription
from edc_registration.models import RegisteredSubject

from meta_pharmacy.constants import METFORMIN
from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin
from meta_subject.models import StudyMedication


class TestStudyMedication(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_visit = self.get_subject_visit()
        self.registered_subject = RegisteredSubject.objects.get(
            subject_identifier=self.subject_visit.subject_identifier
        )

    def test_rx_exists(self):
        self.assertRaises(
            PrescriptionAlreadyExists,
            create_prescription,
            subject_identifier=self.registered_subject.subject_identifier,
            report_datetime=self.registered_subject.registration_datetime,
            medication_names=[METFORMIN],
            site=self.registered_subject.site,
        )

    def test_missing_formulation(self):
        obj = StudyMedication(subject_visit=self.subject_visit)
        with self.assertRaises(StudyMedicationError) as cm:
            obj.save()
        self.assertIn("Formulation", str(cm.exception))

    def test_missing_dosage_guideline(self):
        medication = Medication.objects.get(name=METFORMIN)
        formulation = Formulation.objects.filter(medication=medication)[0]
        obj = StudyMedication(
            subject_visit=self.subject_visit,
            refill_start_datetime=self.subject_visit.report_datetime,
            refill_to_next_visit=YES,
            formulation=formulation,
        )
        with self.assertRaises(StudyMedicationError) as cm:
            obj.save()
        self.assertIn("Dosage guideline", str(cm.exception))

    def test_study_med_longitudinal_and_one_rx_refill_created(self):
        medication = Medication.objects.get(name=METFORMIN)
        formulation = Formulation.objects.filter(medication=medication)[0]
        dosage_guideline = DosageGuideline.objects.filter(medication=medication)[0]
        obj = StudyMedication(
            subject_visit=self.subject_visit,
            refill_start_datetime=self.subject_visit.report_datetime,
            formulation=formulation,
            dosage_guideline=dosage_guideline,
            refill_to_next_visit=YES,
        )
        obj.save()
        try:
            RxRefill.objects.get(
                rx=obj.rx,
                rx__subject_identifier=self.subject_visit.subject_identifier,
                refill_start_datetime=obj.refill_start_datetime,
            )
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised")

        self.assertEqual(RxRefill.objects.filter(rx=obj.rx).count(), 1)

    def test_study_med_longitudinal_updated_and_same_rx_refill_updated(self):
        """Assert study med update operates on same rx_refill when updated"""
        medication = Medication.objects.get(name=METFORMIN)
        formulation = Formulation.objects.filter(medication=medication)[0]
        dosage_guideline = DosageGuideline.objects.filter(medication=medication)[0]
        obj = StudyMedication(
            subject_visit=self.subject_visit,
            refill_start_datetime=self.subject_visit.report_datetime,
            formulation=formulation,
            dosage_guideline=dosage_guideline,
            refill_to_next_visit=YES,
        )
        obj.save()
        obj.save()
        self.assertEqual(RxRefill.objects.filter(rx=obj.rx).count(), 1)
        changed_dosage_guideline = DosageGuideline.objects.filter(medication=medication)[1]
        obj.dosage_guideline = changed_dosage_guideline
        obj.save()
        self.assertEqual(RxRefill.objects.filter(rx=obj.rx).count(), 1)
        rx = RxRefill.objects.get(rx=obj.rx)
        self.assertEqual(rx.dosage_guideline, obj.dosage_guideline)

    def test_study_med_longitudinal_update_start_datetime_and_same_rx_refill_updated(self):
        """Assert study med update operates on same rx_refill when updated"""
        medication = Medication.objects.get(name=METFORMIN)
        formulation = Formulation.objects.filter(medication=medication)[0]
        dosage_guideline = DosageGuideline.objects.filter(medication=medication)[0]
        obj = StudyMedication(
            subject_visit=self.subject_visit,
            refill_start_datetime=self.subject_visit.report_datetime,
            formulation=formulation,
            dosage_guideline=dosage_guideline,
            refill_to_next_visit=YES,
        )
        obj.save()
        obj.save()
        self.assertEqual(RxRefill.objects.filter(rx=obj.rx).count(), 1)
        obj.refill_start_datetime = obj.refill_start_datetime + relativedelta(hours=1)
        obj.save()
        self.assertEqual(RxRefill.objects.filter(rx=obj.rx).count(), 1)
        rx = RxRefill.objects.get(rx=obj.rx)
        self.assertEqual(rx.refill_start_datetime, obj.refill_start_datetime)

    def test_study_med_longitudinal_and_one_rx_refill_created2(self):
        medication = Medication.objects.get(name=METFORMIN)
        formulation = Formulation.objects.filter(medication=medication)[0]
        dosage_guideline = DosageGuideline.objects.filter(medication=medication)[0]
        obj = StudyMedication(
            subject_visit=self.subject_visit,
            refill_start_datetime=self.subject_visit.report_datetime,
            formulation=formulation,
            dosage_guideline=dosage_guideline,
            refill_to_next_visit=YES,
        )
        obj.save()
        try:
            RxRefill.objects.get(
                rx=obj.rx,
                rx__subject_identifier=self.subject_visit.subject_identifier,
                refill_start_datetime=obj.refill_start_datetime,
            )
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised")

        self.assertEqual(RxRefill.objects.filter(rx=obj.rx).count(), 1)

        try:
            RxRefill.objects.get(refill_identifier=obj.refill_identifier)
        except ObjectDoesNotExist:
            self.fail("ObjectDoesNotExist unexpectedly raised")

    def test_study_med_longitudinal_and_two_rx_refill_created(self):
        date_sequence = []
        medication = Medication.objects.get(name=METFORMIN)
        formulation = Formulation.objects.filter(medication=medication)[0]
        dosage_guideline = DosageGuideline.objects.filter(medication=medication)[0]
        subject_visits = [self.subject_visit, self.get_next_subject_visit(self.subject_visit)]
        for i in [1, 2]:
            subject_visit = subject_visits[i - 1]
            obj = StudyMedication(
                subject_visit=subject_visit,
                refill_start_datetime=subject_visit.report_datetime,
                formulation=formulation,
                dosage_guideline=dosage_guideline,
                refill_to_next_visit=YES,
            )
            obj.save()
            date_sequence.append(obj.refill_start_datetime)
            try:
                RxRefill.objects.get(
                    rx=obj.rx,
                    rx__subject_identifier=subject_visit.subject_identifier,
                    refill_start_datetime=obj.refill_start_datetime,
                )
            except ObjectDoesNotExist:
                self.fail("ObjectDoesNotExist unexpectedly raised")

            self.assertEqual(RxRefill.objects.filter(rx=obj.rx).count(), i)

            try:
                rx_refill = RxRefill.objects.get(refill_identifier=obj.refill_identifier)
            except ObjectDoesNotExist:
                self.fail("ObjectDoesNotExist unexpectedly raised")
            date_sequence.append(rx_refill.refill_end_datetime)
        dte1, dte2, dte3, dte4 = date_sequence
        self.assertTrue(dte1 <= dte2 <= dte3 <= dte4)

    def test_study_med_longitudinal(self):
        StudyMedication(
            subject_visit=self.subject_visit,
            refill_start_datetime=self.subject_visit.report_datetime,
            refill_to_next_visit=YES,
        )
        subject_visit = self.get_next_subject_visit(self.subject_visit)
        StudyMedication(
            subject_visit=subject_visit,
            refill_start_datetime=subject_visit.report_datetime,
            refill_to_next_visit=YES,
        )
        subject_visit = self.get_next_subject_visit(subject_visit)
        StudyMedication(
            subject_visit=subject_visit,
            refill_start_datetime=subject_visit.report_datetime,
            refill_to_next_visit=YES,
        )
        subject_visit = self.get_next_subject_visit(subject_visit)
        StudyMedication(
            subject_visit=subject_visit,
            refill_start_datetime=subject_visit.report_datetime,
            refill_to_next_visit=YES,
        )
        subject_visit = self.get_next_subject_visit(subject_visit)
        StudyMedication(
            subject_visit=subject_visit,
            refill_start_datetime=subject_visit.report_datetime,
            refill_to_next_visit=YES,
        )
        subject_visit = self.get_next_subject_visit(subject_visit)
        StudyMedication(
            subject_visit=subject_visit,
            refill_start_datetime=subject_visit.report_datetime,
            refill_to_next_visit=YES,
        )
        subject_visit = self.get_next_subject_visit(subject_visit)
        StudyMedication(
            subject_visit=subject_visit,
            refill_start_datetime=subject_visit.report_datetime,
            refill_to_next_visit=YES,
        )
        subject_visit = self.get_next_subject_visit(subject_visit)
        StudyMedication(
            subject_visit=subject_visit,
            refill_start_datetime=subject_visit.report_datetime,
            refill_to_next_visit=YES,
        )



from typing import Any

from django.db import models
from django.urls import reverse
from django.utils.html import format_html
from edc_constants.constants import NO, TBD, YES
from edc_utils import get_utcnow

from ..constants import EGFR_NOT_CALCULATED
from .eligibility_part_one import EligibilityPartOne
from .eligibility_part_three import EligibilityPartThreePhaseThree
from .eligibility_part_two import EligibilityPartTwo


class SubjectScreeningEligibilityError(Exception):
    pass


def get_eligible_as_word(
    obj=None,
    eligible_part_one=None,
    eligible_part_two=None,
    eligible_part_three=None,
    unsuitable_for_study=None,
    reasons_ineligible=None,
):
    eligible = TBD
    reasons_ineligible = {} if reasons_ineligible is None else reasons_ineligible
    eligible_part_one = obj.eligible_part_one if obj else eligible_part_one
    eligible_part_two = obj.eligible_part_two if obj else eligible_part_two
    eligible_part_three = obj.eligible_part_three if obj else eligible_part_three
    unsuitable_for_study = obj.unsuitable_for_study if obj else unsuitable_for_study

    if unsuitable_for_study == YES:
        reasons_ineligible.update(unsuitable_for_study="Subject unsuitable")
        eligible = NO
    elif all(
        [
            eligible_part_one == YES,
            eligible_part_two == YES,
            eligible_part_three == YES,
        ]
    ):
        eligible = YES
    elif NO in [eligible_part_one, eligible_part_two, eligible_part_three]:
        eligible = NO
    elif TBD in [eligible_part_one, eligible_part_two, eligible_part_three]:
        eligible = TBD
    if eligible == YES and reasons_ineligible:
        raise SubjectScreeningEligibilityError(
            f"Expected reasons_ineligible to be None. Got {reasons_ineligible}."
        )
    return eligible, reasons_ineligible


def get_display_label(obj):
    eligible, _ = get_eligible_as_word(obj)
    if eligible == YES:
        display_label = "ELIGIBLE"
    elif eligible == TBD:
        display_label = "PENDING"
        if EGFR_NOT_CALCULATED in (obj.reasons_ineligible or {}):
            display_label = "PENDING (SCR/eGFR)"
        elif "fbg_ogtt_incomplete" in (obj.reasons_ineligible or {}):
            display_label = "PENDING (FBG/OGTT)"
    else:
        display_label = "not eligible"
    return display_label


class MetaEligibility:
    """A wrapper class for three eligibility classes.

    Determines if a subject is eligible or not.

    Eligibility is assessed in three parts.

    Instantiated in the save() method of the screening proxy models.

    # For example, for part one:
    #
    #     def save(self, *args, **kwargs):
    #         eligibility = Eligibility(self)
    #         try:
    #             eligibility.assess_eligibility_for_part_one()
    #         except EligibilityPartOneError:
    #             pass
    #         eligibility.update_eligibility_fields()
    #         super().save(*args, **kwargs)

    """

    eligibility_values = [YES, NO, TBD]
    default_options = dict(
        eligible_value_default=TBD,
        eligible_values_list=[YES, NO, TBD],
        is_eligible_value=YES,
    )

    def __init__(
        self,
        model_obj: models.Model = None,
        defaults: dict = None,
        update_model=None,
    ):
        self.part_one = None
        self.part_two = None
        self.part_three = None
        self.update_model = True if update_model is None else update_model
        self.eligible = TBD
        self.reasons_ineligible = {}
        self.model_obj = model_obj
        self.default_options = defaults or self.default_options
        self.assess_eligibility_for_all_parts()
        if self.update_model:
            self.update_model_final()

    def __repr__(self: Any) -> str:
        return f"{self.__class__.__name__}()"

    def assess_eligibility_for_all_parts(self: Any):
        eligibility_part_one_cls = EligibilityPartOne
        eligibility_part_two_cls = EligibilityPartTwo
        eligibility_part_three_cls = EligibilityPartThreePhaseThree
        self.part_one = eligibility_part_one_cls(
            model_obj=self.model_obj,
            update_model=self.update_model,
            **self.default_options,
        )
        self.reasons_ineligible.update(**self.part_one.reasons_ineligible)
        self.part_two = eligibility_part_two_cls(
            model_obj=self.model_obj,
            update_model=self.update_model,
            **self.default_options,
        )
        self.reasons_ineligible.update(**self.part_two.reasons_ineligible)
        self.part_three = eligibility_part_three_cls(
            model_obj=self.model_obj,
            update_model=self.update_model,
            **self.default_options,
        )
        self.reasons_ineligible.update(**self.part_three.reasons_ineligible)
        self.check_eligibility_values_or_raise()
        self.eligible, self.reasons_ineligible = get_eligible_as_word(
            eligible_part_one=self.part_one.eligible,
            eligible_part_two=self.part_two.eligible,
            eligible_part_three=self.part_three.eligible,
            reasons_ineligible=self.reasons_ineligible,
            unsuitable_for_study=self.model_obj.unsuitable_for_study,
        )

    def update_model_final(self: Any):
        self.model_obj.reasons_ineligible = "|".join(self.reasons_ineligible)
        self.model_obj.eligible = self.is_eligible
        if self.is_eligible:
            self.model_obj.eligibility_datetime = (
                self.model_obj.part_three_report_datetime or get_utcnow()
            )
        else:
            self.model_obj.eligibility_datetime = None

    @property
    def is_eligible(self: Any) -> bool:
        """Returns True if eligible else False"""
        return True if self.eligible == YES else False

    def check_eligibility_values_or_raise(self: Any):
        for response in [
            self.part_one.eligible,
            self.part_two.eligible,
            self.part_three.eligible,
        ]:
            if response not in self.eligibility_values:
                raise SubjectScreeningEligibilityError(
                    "Invalid value for `eligible`. "
                    f"Expected one of [{self.eligibility_values}]. Got `{response}`."
                )

    @property
    def display_label(self: Any):
        return get_display_label(obj=self.model_obj)

    def eligibility_status(self: Any, add_urls=None):
        if add_urls:
            url_p1 = reverse(
                "meta_screening_admin:meta_screening_screeningpartone_change",
                args=(self.part_one.model_obj.id,),
            )
            url_p2 = reverse(
                "meta_screening_admin:meta_screening_screeningparttwo_change",
                args=(self.part_two.model_obj.id,),
            )
            url_p3 = reverse(
                "meta_screening_admin:meta_screening_screeningpartthree_change",
                args=(self.part_three.model_obj.id,),
            )
            status_str = format_html(
                '<A href="{url_p1}">P1: {p1_eligible}</A>'
                '<BR><A href="{url_p2}">P2: {p2_eligible}</A>'
                '<BR><A href="{url_p3}">P3: {p3_eligible}</A><BR>',
                url_p1=url_p1,
                p1_eligible=self.part_one.eligible.upper(),
                url_p2=url_p2,
                p2_eligible=self.part_two.eligible.upper(),
                url_p3=url_p3,
                p3_eligible=self.part_three.eligible.upper(),
            )
        else:
            status_str = format_html(
                "P1: {p1_eligible}<BR>" "P2: {p2_eligible}<BR>" "P3: {p3_eligible}<BR>",
                p1_eligible=self.part_one.eligible.upper(),
                p2_eligible=self.part_two.eligible.upper(),
                p3_eligible=self.part_three.eligible.upper(),
            )
        display_label = self.display_label
        if "PENDING" in display_label:
            display_label = f'<font color="orange"><B>{display_label}</B></font>'
        return status_str + display_label



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin.history import SimpleHistoryAdmin

from ..admin_site import meta_subject_admin
from ..forms import SubjectVisitMissedForm
from ..models import SubjectVisitMissed
from .modeladmin import CrfModelAdminMixin


@admin.register(SubjectVisitMissed, site=meta_subject_admin)
class SubjectVisitMissedAdmin(
    CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin
):
    form = SubjectVisitMissedForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Contact History",
            {
                "fields": (
                    "survival_status",
                    "contact_attempted",
                    "contact_made",
                    "contact_attempts_count",
                    "contact_attempts_explained",
                    "contact_last_date",
                    "missed_reasons",
                    "missed_reasons_other",
                    "comment",
                ),
            },
        ),
        audit_fieldset_tuple,
    )

    filter_horizontal = ("missed_reasons",)

    radio_fields = {
        "survival_status": admin.VERTICAL,
        "contact_attempted": admin.VERTICAL,
        "contact_made": admin.VERTICAL,
    }



# Generated by Django 5.0.8 on 2024-08-22 19:36

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("meta_reports", "0031_endpointsproxy"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="endpointsproxy",
            options={
                "default_permissions": ("view", "export", "viewallsites"),
                "verbose_name": "Endpoints (DM): All",
                "verbose_name_plural": "Endpoints (DM): All",
            },
        ),
    ]



from .signals import subject_consent_on_post_delete, subject_consent_on_post_save
from .subject_consent import SubjectConsent
from .subject_consent_v1 import SubjectConsentV1
from .subject_consent_v1_ext import SubjectConsentV1Ext
from .subject_reconsent import SubjectReconsent



# Generated by Django 3.0.4 on 2020-04-19 19:23

import django.core.validators
import edc_vitals.models.fields.blood_pressure
import edc_vitals.models.fields.height
import edc_vitals.models.fields.weight
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_subject", "0021_auto_20200417_0332"),
    ]

    operations = [
        migrations.AddField(
            model_name="coronakap",
            name="dia_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="coronakap",
            name="diabetic",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="No",
                max_length=25,
                verbose_name="Does the patient have diabetes?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="coronakap",
            name="height",
            field=edc_vitals.models.fields.height.HeightField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="coronakap",
            name="hiv_pos",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="No",
                max_length=25,
                verbose_name="Does the patient have HIV infection?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="coronakap",
            name="hypertensive",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="No",
                max_length=25,
                verbose_name="Does the patient have hypertension?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="coronakap",
            name="months_on_art",
            field=models.IntegerField(
                blank=True,
                help_text="in months",
                null=True,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="If yes, for how many months has the patient been on antiretroviral therapy?",
            ),
        ),
        migrations.AddField(
            model_name="coronakap",
            name="sys_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="coronakap",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalcoronakap",
            name="dia_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.DiastolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalcoronakap",
            name="diabetic",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="No",
                max_length=25,
                verbose_name="Does the patient have diabetes?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalcoronakap",
            name="height",
            field=edc_vitals.models.fields.height.HeightField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalcoronakap",
            name="hiv_pos",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="No",
                max_length=25,
                verbose_name="Does the patient have HIV infection?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalcoronakap",
            name="hypertensive",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="No",
                max_length=25,
                verbose_name="Does the patient have hypertension?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalcoronakap",
            name="months_on_art",
            field=models.IntegerField(
                blank=True,
                help_text="in months",
                null=True,
                validators=[django.core.validators.MinValueValidator(0)],
                verbose_name="If yes, for how many months has the patient been on antiretroviral therapy?",
            ),
        ),
        migrations.AddField(
            model_name="historicalcoronakap",
            name="sys_blood_pressure",
            field=edc_vitals.models.fields.blood_pressure.SystolicPressureField(
                blank=True, null=True
            ),
        ),
        migrations.AddField(
            model_name="historicalcoronakap",
            name="weight",
            field=edc_vitals.models.fields.weight.WeightField(blank=True, null=True),
        ),
    ]



from dataclasses import dataclass, field

import numpy as np
import pandas as pd
from edc_constants.constants import YES

from ..constants import endpoint_cases


class EndpointTdeltaError(Exception):
    pass


class InvalidCaseList(Exception):
    pass


@dataclass(kw_only=True)
class CaseData:
    df: pd.DataFrame
    index: int
    fbg_value: float | None = field(default=None, init=False)
    fbg_datetime: pd.Timestamp | None = field(default=None, init=False)
    fasted: str | None = field(default=None, init=False)
    ogtt_value: float | None = field(default=None, init=False)
    next_fbg_value: float | None = field(default=None, init=False)
    next_fbg_datetime: pd.Timestamp | None = field(default=None, init=False)
    next_fasted: str | None = field(default=None, init=False)
    next_ogtt_value: float | None = field(default=None, init=False)

    previous_fbg_value: float | None = field(default=None, init=False)
    previous_fbg_datetime: pd.Timestamp | None = field(default=None, init=False)
    previous_fasted: str | None = field(default=None, init=False)
    previous_ogtt_value: float | None = field(default=None, init=False)

    fbg_threshold: float = field(default=7.0, init=False)
    ogtt_threshold: float = field(default=11.1, init=False)

    def __post_init__(self):
        self.fbg_value = self.df.loc[self.index, "fbg_value"]
        self.fbg_datetime = self.df.loc[self.index, "fbg_datetime"]
        self.ogtt_value = self.df.loc[self.index, "ogtt_value"]
        self.fasted = self.df.loc[self.index, "fasted"]

        try:
            self.next_fbg_value = self.df.loc[self.index + 1, "fbg_value"]
        except KeyError:
            self.next_fbg_value = np.nan
            self.next_fbg_datetime = pd.NaT
            self.next_ogtt_value = np.nan
            self.next_fasted = np.nan
        else:
            self.next_fbg_datetime = self.df.loc[self.index + 1, "fbg_datetime"]
            self.next_ogtt_value = self.df.loc[self.index + 1, "ogtt_value"]
            self.next_fasted = self.df.loc[self.index + 1, "fasted"]

        try:
            self.previous_fbg_value = self.df.loc[self.index - 1, "fbg_value"]
        except KeyError:
            self.previous_fbg_value = np.nan
            self.previous_fbg_datetime = pd.NaT
            self.previous_ogtt_value = np.nan
            self.previous_fasted = np.nan
        else:
            self.previous_fbg_datetime = self.df.loc[self.index - 1, "fbg_datetime"]
            self.previous_ogtt_value = self.df.loc[self.index - 1, "ogtt_value"]
            self.previous_fasted = self.df.loc[self.index - 1, "fasted"]

    def case_two(self) -> bool:
        """ "FBG >= 7 x 2, first OGTT<=11.1"""
        if (
            self.fbg_value >= self.fbg_threshold
            and self.next_fbg_value >= self.fbg_threshold
            and 0.0 < self.ogtt_value < self.ogtt_threshold
            and self.fasted == YES
            and self.next_fasted == YES
            and (self.next_fbg_datetime.date() - self.fbg_datetime.date()).days > 6
        ):
            return True
        return False

    def case_three(self) -> bool:
        """ "FBG >= 7 x 2, second OGTT<=11.1"""
        if (
            self.fbg_value >= self.fbg_threshold
            and self.next_fbg_value >= self.fbg_threshold
            and 0.0 < self.next_ogtt_value < self.ogtt_threshold
            and self.fasted == YES
            and self.next_fasted == YES
            and (self.next_fbg_datetime.date() - self.fbg_datetime.date()).days > 6
        ):
            return True
        return False

    def case_two_reversed(self) -> bool:
        """Same as case 2, but with the previous FBG reading."""
        if (
            self.fbg_value >= self.fbg_threshold
            and self.previous_fbg_value >= self.fbg_threshold
            and 0.0 < self.previous_ogtt_value < self.ogtt_threshold
            and self.fasted == YES
            and self.previous_fasted == YES
            and (self.fbg_datetime.date() - self.previous_fbg_datetime.date()).days > 6
        ):
            return True
        return False


class EndpointByDate:
    """Given all timepoints for a subject, flag the first timepoint
    where the protocol endpoint is reached.

    IMPORTANT: Remove case one before passing to this class
        * case 1. any OGTT >= 11.1

    Evaluation is done in order

    Order of protocol endpoint evaluation:
      * case 2. FBG >= 7 x 2, first OGTT<11.1
      * case 3.  FBG >= 7 x 2, second OGTT<11.1

    Additional criteria considered:
      1. any threshhold FBG must be taken while fasted (fasted=YES)
      2. threshhold FBG readings must be consecutive (no
         readings below threshold in the sequence regardless
         of fasting)
      3. at least 7 days between threshhold FBG readings.
      4. at least one of the two threshold FBG readings must be taken
         with an OGTT at the same timepoint.
    """

    def __init__(
        self,
        subject_df: pd.DataFrame = None,
        fbg_threshhold: float = None,
        ogtt_threshhold: float = None,
    ):
        self.row = None
        self.index = None
        self.subject_df = subject_df.sort_values(by=["visit_code"]).reset_index(drop=True)
        self.fbg_threshhold = fbg_threshhold
        self.ogtt_threshhold = ogtt_threshhold
        self.evaluate()

    def evaluate(self):
        for index, _ in self.subject_df.iterrows():
            case_data = CaseData(df=self.subject_df, index=index)
            if case_data.case_two():
                self.endpoint_reached(index, case=2, fbg_datetime=case_data.next_fbg_datetime)
                break
            elif case_data.case_three():
                self.endpoint_reached(index, case=3, fbg_datetime=case_data.next_fbg_datetime)
                break
            elif case_data.case_two_reversed():
                self.endpoint_reached(index, case=2, fbg_datetime=case_data.fbg_datetime)
                break
            else:
                pass

    def endpoint_reached(self, index: int, case: int, fbg_datetime: pd.Timestamp):
        """Update the subject_df"""
        self.subject_df.loc[self.subject_df["fbg_datetime"] == fbg_datetime, "endpoint"] = 1
        self.subject_df["interval_in_days"] = np.nan
        self.subject_df.loc[
            self.subject_df["fbg_datetime"] == fbg_datetime, "endpoint_type"
        ] = case
        self.subject_df.loc[
            self.subject_df["fbg_datetime"] == fbg_datetime, "endpoint_label"
        ] = endpoint_cases[case]



from django.test import TestCase
from edc_constants.constants import FEMALE

from meta_screening.tests.meta_test_case_mixin import MetaTestCaseMixin


class TestIncidence(MetaTestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_visit = self.get_subject_visit(gender=FEMALE)



from django.apps import apps as django_apps
from django.test import TestCase
from edc_sites.managers import CurrentSiteManager


class TestManagers(TestCase):
    def test_models(self):
        app_config = django_apps.get_app_config("meta_prn")
        for model_cls in app_config.get_models():
            if "historical" not in model_cls._meta.label_lower:
                self.assertFalse(
                    isinstance(model_cls._default_manager, (CurrentSiteManager,)),
                    msg=model_cls._meta.label_lower,
                )



# Generated by Django 3.2.11 on 2022-03-17 22:33

import uuid

import _socket
import django.db.models.deletion
import django.db.models.manager
import django_audit_fields.fields.hostname_modification_field
import django_audit_fields.fields.userfield
import django_audit_fields.fields.uuid_auto_field
import django_audit_fields.models.audit_model_mixin
import django_revision.revision_field
import edc_identifier.managers
import edc_model.validators.date
import edc_protocol.validators
import edc_sites.models
import edc_utils.date
import simple_history.models
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("edc_action_item", "0028_auto_20210203_0706"),
        ("sites", "0002_alter_domain_unique"),
        ("meta_prn", "0021_auto_20220316_2147"),
    ]

    operations = [
        migrations.CreateModel(
            name="HistoricalOffSchedulePostnatal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(db_index=True, max_length=50)),
                ("tracking_identifier", models.CharField(db_index=True, max_length=32)),
                ("action_identifier", models.CharField(db_index=True, max_length=50)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("report_datetime", models.DateTimeField(editable=False)),
                (
                    "offschedule_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Date and time subject taken off schedule",
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical off schedule postnatal",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="HistoricalOnSchedulePostnatal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        db_index=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                    ),
                ),
                ("subject_identifier", models.CharField(db_index=True, max_length=50)),
                ("report_datetime", models.DateTimeField(editable=False)),
                (
                    "onschedule_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                    ),
                ),
                (
                    "history_id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        editable=False,
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("history_date", models.DateTimeField()),
                ("history_change_reason", models.CharField(max_length=100, null=True)),
                (
                    "history_type",
                    models.CharField(
                        choices=[("+", "Created"), ("~", "Changed"), ("-", "Deleted")],
                        max_length=1,
                    ),
                ),
                (
                    "history_user",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        blank=True,
                        db_constraint=False,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "verbose_name": "historical on schedule postnatal",
                "ordering": ("-history_date", "-history_id"),
                "get_latest_by": "history_date",
            },
            bases=(simple_history.models.HistoricalChanges, models.Model),
        ),
        migrations.CreateModel(
            name="OffSchedulePostnatal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                ("tracking_identifier", models.CharField(max_length=32, unique=True)),
                ("action_identifier", models.CharField(max_length=50, unique=True)),
                (
                    "parent_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to parent reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                (
                    "related_action_identifier",
                    models.CharField(
                        blank=True,
                        help_text="action identifier that links to related reference model instance.",
                        max_length=30,
                        null=True,
                    ),
                ),
                ("action_item_reason", models.TextField(editable=False, null=True)),
                ("report_datetime", models.DateTimeField(editable=False)),
                (
                    "offschedule_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                        verbose_name="Date and time subject taken off schedule",
                    ),
                ),
                (
                    "action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "parent_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "related_action_item",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="edc_action_item.actionitem",
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            managers=[
                ("objects", django.db.models.manager.Manager()),
                (
                    "on_site",
                    edc_sites.models.CurrentSiteManager(),
                ),
            ],
        ),
        migrations.CreateModel(
            name="OnSchedulePostnatal",
            fields=[
                (
                    "revision",
                    django_revision.revision_field.RevisionField(
                        blank=True,
                        editable=False,
                        help_text="System field. Git repository tag:branch:commit.",
                        max_length=75,
                        null=True,
                        verbose_name="Revision",
                    ),
                ),
                (
                    "created",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "modified",
                    models.DateTimeField(
                        blank=True,
                        default=django_audit_fields.models.audit_model_mixin.utcnow,
                    ),
                ),
                (
                    "user_created",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user created",
                    ),
                ),
                (
                    "user_modified",
                    django_audit_fields.fields.userfield.UserField(
                        blank=True,
                        help_text="Updated by admin.save_model",
                        max_length=50,
                        verbose_name="user modified",
                    ),
                ),
                (
                    "hostname_created",
                    models.CharField(
                        blank=True,
                        default=_socket.gethostname,
                        help_text="System field. (modified on create only)",
                        max_length=60,
                    ),
                ),
                (
                    "hostname_modified",
                    django_audit_fields.fields.hostname_modification_field.HostnameModificationField(
                        blank=True,
                        help_text="System field. (modified on every save)",
                        max_length=50,
                    ),
                ),
                ("device_created", models.CharField(blank=True, max_length=10)),
                ("device_modified", models.CharField(blank=True, max_length=10)),
                (
                    "id",
                    django_audit_fields.fields.uuid_auto_field.UUIDAutoField(
                        blank=True,
                        editable=False,
                        help_text="System auto field. UUID primary key.",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("subject_identifier", models.CharField(max_length=50, unique=True)),
                ("report_datetime", models.DateTimeField(editable=False)),
                (
                    "onschedule_datetime",
                    models.DateTimeField(
                        default=edc_utils.date.get_utcnow,
                        validators=[
                            edc_protocol.validators.datetime_not_before_study_start,
                            edc_model.validators.date.datetime_not_future,
                        ],
                    ),
                ),
                (
                    "site",
                    models.ForeignKey(
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="+",
                        to="sites.site",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            managers=[
                (
                    "on_site",
                    edc_sites.models.CurrentSiteManager(),
                ),
                ("objects", edc_identifier.managers.SubjectIdentifierManager()),
            ],
        ),
        migrations.AlterUniqueTogether(
            name="delivery",
            unique_together=None,
        ),
        migrations.RemoveField(
            model_name="delivery",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="delivery",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="delivery",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="delivery",
            name="site",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="delivery",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="historicalbirthoutcomes",
            name="site",
        ),
        migrations.RemoveField(
            model_name="historicaldelivery",
            name="action_item",
        ),
        migrations.RemoveField(
            model_name="historicaldelivery",
            name="history_user",
        ),
        migrations.RemoveField(
            model_name="historicaldelivery",
            name="parent_action_item",
        ),
        migrations.RemoveField(
            model_name="historicaldelivery",
            name="related_action_item",
        ),
        migrations.RemoveField(
            model_name="historicaldelivery",
            name="site",
        ),
        migrations.DeleteModel(
            name="BirthOutcomes",
        ),
        migrations.DeleteModel(
            name="Delivery",
        ),
        migrations.DeleteModel(
            name="HistoricalBirthOutcomes",
        ),
        migrations.DeleteModel(
            name="HistoricalDelivery",
        ),
        migrations.AddIndex(
            model_name="onschedulepostnatal",
            index=models.Index(
                fields=["id", "subject_identifier", "onschedule_datetime", "site"],
                name="meta_prn_on_id_e5ef52_idx",
            ),
        ),
        migrations.AddIndex(
            model_name="offschedulepostnatal",
            index=models.Index(
                fields=["id", "subject_identifier", "offschedule_datetime", "site"],
                name="meta_prn_of_id_3c0260_idx",
            ),
        ),
    ]



# Generated by Django 3.0.6 on 2020-06-17 14:38

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("meta_lists", "0009_auto_20200613_2041"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="subjectvisitmissedreasons",
            options={
                "default_permissions": (
                    "add",
                    "change",
                    "delete",
                    "view",
                    "export",
                    "import",
                ),
                "ordering": ["display_index", "display_name"],
                "verbose_name": "Subject Missed Visit Reasons",
                "verbose_name_plural": "Subject Missed Visit Reasons",
            },
        ),
        migrations.RemoveIndex(
            model_name="subjectvisitmissedreasons",
            name="meta_lists__id_50745c_idx",
        ),
        migrations.AddIndex(
            model_name="subjectvisitmissedreasons",
            index=models.Index(
                fields=["id", "display_name", "display_index"],
                name="meta_lists__id_4ee3a4_idx",
            ),
        ),
    ]
